// Graphics3DView.cpp : implementation of the CGraphics3DView class
//

#include "stdafx.h"
#include "Graphics3D.h"

#include "Graphics3DDoc.h"
#include "Graphics3DView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

#include <math.h>
#include "TransformDlg.h"

// CGraphics3DView

IMPLEMENT_DYNCREATE(CGraphics3DView, CView)

BEGIN_MESSAGE_MAP(CGraphics3DView, CView)
    ON_COMMAND(ID_TRANSFORM_ANIMATE, OnTransformAnimate)
    ON_COMMAND(ID_TRANSFORM_ROTATE, OnTransformRotate)
    ON_COMMAND(ID_TRANSFORM_SCALE, OnTransformScale)
    ON_COMMAND(ID_TRANSFORM_TRANSLATE, OnTransformTranslate)
    ON_UPDATE_COMMAND_UI(ID_TRANSFORM_ANIMATE, OnUpdateTransformAnimate)
    ON_WM_RBUTTONDOWN()
    ON_WM_TIMER()
    ON_WM_DESTROY()
END_MESSAGE_MAP()

// CGraphics3DView construction/destruction

CGraphics3DView::CGraphics3DView()
{
	// TODO: add construction code here
    InitModel();

    m_xRotate = 0;
    m_yRotate = 0;
    m_zRotate = 0;
    m_xScale = 1;
    m_yScale = 1;
    m_zScale = 1;
    m_xTranslate = 0;
    m_yTranslate = 0;
    m_zTranslate = 0;

    TransformModel();

    m_animate = FALSE;
}

CGraphics3DView::~CGraphics3DView()
{
}

BOOL CGraphics3DView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

// CGraphics3DView drawing

void CGraphics3DView::OnDraw(CDC* pDC)
{
	CGraphics3DDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// TODO: add draw code for native data here
    DrawModel(pDC, m_cube);
}


// CGraphics3DView diagnostics

#ifdef _DEBUG
void CGraphics3DView::AssertValid() const
{
	CView::AssertValid();
}

void CGraphics3DView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CGraphics3DDoc* CGraphics3DView::GetDocument() const // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CGraphics3DDoc)));
	return (CGraphics3DDoc*)m_pDocument;
}
#endif //_DEBUG


// CGraphics3DView message handlers

void CGraphics3DView::OnTransformAnimate()
{
    // TODO: Add your command handler code here
    if (m_animate)
    {
        KillTimer(1);
        m_animate = FALSE;
    }
    else
    {
        SetTimer(1, 50, NULL);
        m_animate = TRUE;
    }
}

void CGraphics3DView::OnTransformRotate()
{
    // TODO: Add your command handler code here
    CTransformDlg dlg;
    dlg.m_xAxis = 0;
    dlg.m_yAxis = 0;
    dlg.m_zAxis = 0;

    int response = dlg.DoModal();

    if (response == IDOK)
    {
        m_xRotate += (int) dlg.m_xAxis;
        m_yRotate += (int) dlg.m_yAxis;
        m_zRotate += (int) dlg.m_zAxis;
    }
}

void CGraphics3DView::OnTransformScale()
{
    // TODO: Add your command handler code here
    CTransformDlg dlg;
    dlg.m_xAxis = 1;
    dlg.m_yAxis = 1;
    dlg.m_zAxis = 1;

    int response = dlg.DoModal();

    if (response == IDOK)
    {
        m_xScale *= dlg.m_xAxis;
        m_yScale *= dlg.m_yAxis;
        m_zScale *= dlg.m_zAxis;
    }
}

void CGraphics3DView::OnTransformTranslate()
{
    // TODO: Add your command handler code here
    CTransformDlg dlg;
    dlg.m_xAxis = 0;
    dlg.m_yAxis = 0;
    dlg.m_zAxis = 0;

    int response = dlg.DoModal();

    if (response == IDOK)
    {
        m_xTranslate += dlg.m_xAxis;
        m_yTranslate += dlg.m_yAxis;
        m_zTranslate += dlg.m_zAxis;
    }
}

void CGraphics3DView::OnUpdateTransformAnimate(CCmdUI *pCmdUI)
{
    // TODO: Add your command update UI handler code here
    if (m_animate)
        pCmdUI->SetCheck(TRUE);
    else
        pCmdUI->SetCheck(FALSE);
}

void CGraphics3DView::OnRButtonDown(UINT nFlags, CPoint point)
{
    // TODO: Add your message handler code here and/or call default
    InitModel();
    TransformModel();
    Invalidate(TRUE);

    CView::OnRButtonDown(nFlags, point);
}

void CGraphics3DView::OnTimer(UINT nIDEvent)
{
    // TODO: Add your message handler code here and/or call default
    m_xRotate += 5;
    m_yRotate += 5;
    m_zRotate += 5;
    InitModel();
    TransformModel();
    Invalidate(TRUE);

    CView::OnTimer(nIDEvent);
}

void CGraphics3DView::OnDestroy()
{
    CView::OnDestroy();

    // TODO: Add your message handler code here
    KillTimer(1);
}

void CGraphics3DView::InitModel()
{
    VERTEX cubeVerts[8] =
        {0,   100, 0,    1,
         100, 100, 0,    1,
         100, 100, -100, 1,
         0,   100, -100, 1,
         0,   0,   0,    1,
         100, 0,   0,    1,
         100, 0,   -100, 1,
         0,   0,   -100, 1};

    EDGE cubeEdges[14] =
        {1, 2,
         2, 3,
         3, 4,
         4, 1,
         5, 6,
         6, 7,
         7, 8,
         8, 5,
         5, 1,
         6, 2,
         7, 3,
         8, 4,
         3, 8,
         4, 7};

    memcpy(m_cubeVerts, cubeVerts, sizeof(m_cubeVerts));
    memcpy(m_cubeEdges, cubeEdges, sizeof(m_cubeEdges));
    m_cube.numVerts = 8;
    m_cube.vertices = m_cubeVerts;
    m_cube.numEdges = 14;
    m_cube.edges = m_cubeEdges;
}

void CGraphics3DView::TransformModel()
{
    MATRIX4X4 m;
    InitMatrix(m);

    Translate(m, (int) m_xTranslate,
        (int) m_yTranslate, (int) m_zTranslate);
    Scale(m, m_xScale, m_yScale, m_zScale);
    RotateX(m, m_xRotate);
    RotateY(m, m_yRotate);
    RotateZ(m, m_zRotate);
    Transform(m_cube, m);
    PerspProject(m_cube, 300);
}

void CGraphics3DView::DrawModel(CDC* pDC, MODEL& model)
{
    int newX, newY;
    RECT clientRect;

    GetClientRect(&clientRect);
    int maxY = clientRect.bottom;

    for (UINT i=0; i<model.numEdges; ++i)
    {
        UINT vertNum = model.edges[i].vertex1;
        newX = model.vertices[vertNum-1].x;
        newY = maxY - model.vertices[vertNum-1].y - 1;
        pDC->MoveTo(newX, newY);

        vertNum = model.edges[i].vertex2;
        newX = model.vertices[vertNum-1].x;
        newY = maxY - model.vertices[vertNum-1].y - 1;
        pDC->LineTo(newX, newY);
    }
}

void CGraphics3DView::PerspProject(MODEL& model, double eye)
{
    for (UINT i=0; i<model.numVerts; ++i)
    {
        int xCoord = model.vertices[i].x;
        int yCoord = model.vertices[i].y;
        int zCoord = model.vertices[i].z;
        double t = 1.0 / (1.0 - zCoord / eye);
        model.vertices[i].x = (int) (xCoord * t);
        model.vertices[i].y = (int) (yCoord * t);
    }
}

void CGraphics3DView::InitMatrix(MATRIX4X4& m)
{
    m[0][0]=1; m[0][1]=0; m[0][2]=0; m[0][3]=0;
    m[1][0]=0; m[1][1]=1; m[1][2]=0; m[1][3]=0;
    m[2][0]=0; m[2][1]=0; m[2][2]=1; m[2][3]=0;
    m[3][0]=0; m[3][1]=0; m[3][2]=0; m[3][3]=1;
}

void CGraphics3DView::CopyMatrix(MATRIX4X4& dst, MATRIX4X4& src)
{
   for (int i=0; i<4; ++i)
      for (int j=0; j<4; ++j)
         dst[i][j] = src[i][j];
}

void CGraphics3DView::MultMatrix(MATRIX4X4& product,
      MATRIX4X4& matrix1, MATRIX4X4& matrix2)
{
    for (int x=0; x<4; ++x)
        for (int y=0; y<4; ++y)
        {
            double sum = 0;
            for (int z=0; z<4; ++z)
                sum += matrix1[x][z] * matrix2[z][y];
            product[x][y] = sum;
        }
}

void CGraphics3DView::Translate(MATRIX4X4& m,
    int xTrans, int yTrans, int zTrans)
{
    MATRIX4X4 m1, m2;

    m1[0][0]=1;      m1[0][1]=0;      m1[0][2]=0;      m1[0][3]=0;
    m1[1][0]=0;      m1[1][1]=1;      m1[1][2]=0;      m1[1][3]=0;
    m1[2][0]=0;      m1[2][1]=0;      m1[2][2]=1;      m1[2][3]=0;
    m1[3][0]=xTrans; m1[3][1]=yTrans; m1[3][2]=zTrans; m1[3][3]=1;

    MultMatrix(m2, m1, m);
    CopyMatrix(m, m2);
}

void CGraphics3DView::Scale(MATRIX4X4& m,
    double xScale, double yScale, double zScale)
{
    MATRIX4X4 m1, m2;

    m1[0][0]=xScale; m1[0][1]=0;      m1[0][2]=0;      m1[0][3]=0;
    m1[1][0]=0;      m1[1][1]=yScale; m1[1][2]=0;      m1[1][3]=0;
    m1[2][0]=0;      m1[2][1]=0;      m1[2][2]=zScale; m1[2][3]=0;
    m1[3][0]=0;      m1[3][1]=0;      m1[3][2]=0;      m1[3][3]=1;

    MultMatrix(m2, m1, m);
    CopyMatrix(m, m2);
}

void CGraphics3DView::RotateX(MATRIX4X4& m, int xAngle)
{
    MATRIX4X4 m1, m2;

    if (xAngle == 0) return;

    double radians = 6.283185308 / (360.0 / xAngle);
    double c = cos(radians);
    double s = sin(radians);

    m1[0][0]=1;  m1[0][1]=0;  m1[0][2]=0; m1[0][3]=0;
    m1[1][0]=0;  m1[1][1]=c;  m1[1][2]=s; m1[1][3]=0;
    m1[2][0]=0;  m1[2][1]=-s; m1[2][2]=c; m1[2][3]=0;
    m1[3][0]=0;  m1[3][1]=0;  m1[3][2]=0; m1[3][3]=1;

    MultMatrix(m2, m1, m);
    CopyMatrix(m, m2);
}

void CGraphics3DView::RotateY(MATRIX4X4& m, int yAngle)
{
    MATRIX4X4 m1, m2;

    if (yAngle == 0) return;

    double radians = 6.283185308 / (360.0 / yAngle);
    double c = cos(radians);
    double s = sin(radians);

    m1[0][0]=c;  m1[0][1]=0;  m1[0][2]=-s; m1[0][3]=0;
    m1[1][0]=0;  m1[1][1]=1;  m1[1][2]=0;  m1[1][3]=0;
    m1[2][0]=s;  m1[2][1]=0;  m1[2][2]=c;  m1[2][3]=0;
    m1[3][0]=0;  m1[3][1]=0;  m1[3][2]=0;  m1[3][3]=1;

    MultMatrix(m2, m1, m);
    CopyMatrix(m, m2);
}

void CGraphics3DView::RotateZ(MATRIX4X4& m, int zAngle)
{
    MATRIX4X4 m1, m2;

    if (zAngle == 0) return;

    double radians = 6.283185308 / (360.0 / zAngle);
    double c = cos(radians);
    double s = sin(radians);

    m1[0][0]=c;  m1[0][1]=s; m1[0][2]=0; m1[0][3]=0;
    m1[1][0]=-s; m1[1][1]=c; m1[1][2]=0; m1[1][3]=0;
    m1[2][0]=0;  m1[2][1]=0; m1[2][2]=1; m1[2][3]=0;
    m1[3][0]=0;  m1[3][1]=0; m1[3][2]=0; m1[3][3]=1;

    MultMatrix(m2, m1, m);
    CopyMatrix(m, m2);
}

void CGraphics3DView::Transform(MODEL& shape, MATRIX4X4& m)
{
    int transformedX, transformedY, transformedZ;

    for (UINT i=0; i<shape.numVerts; ++i)
    {
        transformedX = (int) (shape.vertices[i].x * m[0][0] +
            shape.vertices[i].y * m[1][0] +
            shape.vertices[i].z * m[2][0] + m[3][0]);
        transformedY = (int) (shape.vertices[i].x * m[0][1] +
            shape.vertices[i].y * m[1][1] +
            shape.vertices[i].z * m[2][1] + m[3][1]);
        transformedZ = (int) (shape.vertices[i].x * m[0][2] +
            shape.vertices[i].y * m[1][2] +
            shape.vertices[i].z * m[2][2] + m[3][2]);
        shape.vertices[i].x = transformedX;
        shape.vertices[i].y = transformedY;
        shape.vertices[i].z = transformedZ;
   }
}
