// Graphics3DView.h : interface of the CGraphics3DView class
//


#pragma once

typedef double MATRIX4X4[4][4];

typedef struct vertex
{
    int x, y, z, w;
} VERTEX;

typedef struct edge
{
    UINT vertex1, vertex2;
} EDGE;

typedef struct model
{
    UINT numVerts;
    VERTEX* vertices;
    UINT numEdges;
    EDGE* edges;
} MODEL;

class CGraphics3DView : public CView
{
protected: // create from serialization only
	CGraphics3DView();
	DECLARE_DYNCREATE(CGraphics3DView)

// Attributes
public:
	CGraphics3DDoc* GetDocument() const;

protected:
    VERTEX m_cubeVerts[8];
    EDGE m_cubeEdges[14];
    MODEL m_cube;
    int m_xRotate, m_yRotate, m_zRotate;
    double m_xScale, m_yScale, m_zScale;
    double m_xTranslate, m_yTranslate, m_zTranslate;
    BOOL m_animate;

// Operations
public:

// Overrides
	public:
	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
protected:

// Implementation
public:
	virtual ~CGraphics3DView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:
    void InitModel();
    void TransformModel();
    void DrawModel(CDC* pDC, MODEL& model);
    void PerspProject(MODEL& model, double eye);
    void Translate(MATRIX4X4& m,
        int xTrans, int yTrans, int zTrans);
    void Scale(MATRIX4X4& m,
        double xScale, double yScale, double zScale);
    void RotateX(MATRIX4X4& m, int xAngle);
    void RotateY(MATRIX4X4& m, int yAngle);
    void RotateZ(MATRIX4X4& m, int zAngle);
    void Transform(MODEL& model, MATRIX4X4& m);
    void MultMatrix(MATRIX4X4& product,
        MATRIX4X4& matrix1, MATRIX4X4& matrix2);
    void InitMatrix(MATRIX4X4& m);
    void CopyMatrix(MATRIX4X4& dst, MATRIX4X4& src);

// Generated message map functions
protected:
	DECLARE_MESSAGE_MAP()
public:
    afx_msg void OnTransformAnimate();
    afx_msg void OnTransformRotate();
    afx_msg void OnTransformScale();
    afx_msg void OnTransformTranslate();
    afx_msg void OnUpdateTransformAnimate(CCmdUI *pCmdUI);
    afx_msg void OnRButtonDown(UINT nFlags, CPoint point);
    afx_msg void OnTimer(UINT nIDEvent);
    afx_msg void OnDestroy();
};

#ifndef _DEBUG  // debug version in Graphics3DView.cpp
inline CGraphics3DDoc* CGraphics3DView::GetDocument() const
   { return reinterpret_cast<CGraphics3DDoc*>(m_pDocument); }
#endif

