//////////////////////////////////////////////////////
// CopyRectsApp.cpp
//////////////////////////////////////////////////////

#include <windows.h>
#include <d3d9.h>
#include <d3dx9tex.h>
#include <dxerr9.h>

// Constants.
const COLUMNSINRECTFILE = 3;
const RECTSIZE = 128;
const DSTCOLUMNCOUNT = 3;
const DSTROWCOUNT = 2;
const XOFFSET = 128;
const YOFFSET = 112;

// Function prototypes.
LRESULT WINAPI WndProc(HWND hWnd, UINT msg,
    WPARAM wParam, LPARAM lParam);
void RegisterWindowClass(HINSTANCE hInstance);
void CreateAppWindow(HINSTANCE hInstance);
WPARAM StartMessageLoop();
HRESULT InitDirect3D();
void Render();
void CleanUpDirect3D();
void PaintBackground();
int ColumnRow2RectNum(int iCol, int iRow, int iNumCols);
HRESULT PlaceRectangle(IDirect3DSurface9* pBackBuffer,
    IDirect3DSurface9* pSrcSurface, int iRectNumber,
    int iDstCol, int iDstRow, int iNumRectCols, int iNumDstCols,
    int iRectSize, int iXOffset, int iYOffset);
int RectNumber2SourceX(int iRectNumber, int iNumCols, int iRectSize);
int RectNumber2SourceY(int iRectNumber, int iNumCols, int iRectSize);
int Column2X(int iCol, int iRectSize, int iNumCols);
int Row2Y(int iRow, int iRectSize);

// Global variables.
HWND g_hWnd;
IDirect3D9* g_pDirect3D = NULL;
IDirect3DDevice9* g_pDirect3DDevice = NULL;
IDirect3DSurface9* g_pBitmapSurface = NULL;
HRESULT g_hResult = D3D_OK;
char g_szErrorMsg[256];
IDirect3DSurface9* g_pBackBuffer = NULL;

//////////////////////////////////////////////////////
// WinMain()
//////////////////////////////////////////////////////
INT WINAPI WinMain(HINSTANCE hInstance, HINSTANCE, LPSTR, INT)
{
    RegisterWindowClass(hInstance);
    CreateAppWindow(hInstance);
    ShowWindow(g_hWnd, SW_SHOWDEFAULT);
    UpdateWindow(g_hWnd);
    HRESULT hResult = InitDirect3D();
    if (SUCCEEDED(hResult))
        WPARAM result = StartMessageLoop();
    CleanUpDirect3D();
    CloseWindow(g_hWnd);
    if (g_hResult != D3D_OK)
        DXTRACE_ERR(g_szErrorMsg, g_hResult);
    return 0;
}

//////////////////////////////////////////////////////
// WndProc()
//////////////////////////////////////////////////////
LRESULT WINAPI WndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    switch(msg)
    {
    case WM_CREATE:
        return 0;

    case WM_DESTROY:
        PostQuitMessage( 0 );
        return 0;

    case WM_PAINT:
        ValidateRect(g_hWnd, NULL);
        return 0;

    case WM_KEYDOWN:
        switch(wParam)
        {
        case VK_ESCAPE:
            PostQuitMessage(WM_QUIT);
            break;
        }
    }
    return DefWindowProc(hWnd, msg, wParam, lParam);
}

//////////////////////////////////////////////////////
// RegisterWindowClass()
//////////////////////////////////////////////////////
void RegisterWindowClass(HINSTANCE hInstance)
{
    WNDCLASSEX wc;
    wc.cbSize = sizeof(WNDCLASSEX);
    wc.style = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
    wc.lpfnWndProc = WndProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0; 
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
    wc.hCursor = (HCURSOR)LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName = NULL;
    wc.lpszClassName = "CopyRectsApp";
    wc.hIconSm = NULL;

    RegisterClassEx(&wc);
}

//////////////////////////////////////////////////////
// CreateAppWindow()
//////////////////////////////////////////////////////
void CreateAppWindow(HINSTANCE hInstance)
{
    g_hWnd = CreateWindowEx(
        NULL,
        "CopyRectsApp",
        "Copy Rectangles Application", 
        WS_OVERLAPPEDWINDOW,
        100,
        100,
        648,
        514,
        GetDesktopWindow(),
        NULL,
        hInstance,
        NULL);
}

//////////////////////////////////////////////////////
// StartMessageLoop()
//////////////////////////////////////////////////////
WPARAM StartMessageLoop()
{
    MSG msg;
    while(1)
    {
        if (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
        {
            if (msg.message == WM_QUIT)
                break;
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
        else
        {
            // Use idle time here.
            Render();
        }
    }
    return msg.wParam;
}

//////////////////////////////////////////////////////
// InitDirect3D()
//////////////////////////////////////////////////////
HRESULT InitDirect3D()
{
    g_pDirect3D = Direct3DCreate9(D3D_SDK_VERSION);
    if (g_pDirect3D == NULL)
    {
        MessageBox(g_hWnd,
            "Couldn't create DirectX object.",
            "DirectX Error", MB_OK);
        return E_FAIL;
    }

    HRESULT hResult = g_pDirect3D->CheckDeviceType(D3DADAPTER_DEFAULT, 
        D3DDEVTYPE_HAL, D3DFMT_X8R8G8B8, D3DFMT_X8R8G8B8, FALSE);
    if (hResult != D3D_OK)
    {
        MessageBox(g_hWnd,
            "Sorry. This program won't\nrun on your system.",
            "DirectX Error", MB_OK);
        return E_FAIL;
    }

    D3DPRESENT_PARAMETERS D3DPresentParams;
    ZeroMemory(&D3DPresentParams, sizeof(D3DPRESENT_PARAMETERS));
    D3DPresentParams.Windowed = FALSE;
    D3DPresentParams.BackBufferCount = 1;
    D3DPresentParams.BackBufferWidth = 640;
    D3DPresentParams.BackBufferHeight = 480;
    D3DPresentParams.BackBufferFormat = D3DFMT_X8R8G8B8;
    D3DPresentParams.SwapEffect = D3DSWAPEFFECT_DISCARD;
    D3DPresentParams.hDeviceWindow = g_hWnd;
    hResult = g_pDirect3D->CreateDevice(D3DADAPTER_DEFAULT, 
        D3DDEVTYPE_HAL, g_hWnd, D3DCREATE_SOFTWARE_VERTEXPROCESSING,
        &D3DPresentParams, &g_pDirect3DDevice);
    if (FAILED(hResult))
    {
        MessageBox(g_hWnd,
            "Failed to create Direct3D device.",
            "DirectX Error", MB_OK);
        return E_FAIL;
    }

    g_hResult = g_pDirect3DDevice->CreateOffscreenPlainSurface(384, 256, 
        D3DFMT_X8R8G8B8, D3DPOOL_DEFAULT, &g_pBitmapSurface, NULL);
    if (FAILED(g_hResult))
    {
        strcpy(g_szErrorMsg, "Error creating bitmap surface.");
        PostQuitMessage(WM_QUIT);
    }
    g_hResult = D3DXLoadSurfaceFromFile(g_pBitmapSurface, NULL, NULL,
        "Image02.bmp", NULL, D3DX_DEFAULT, 0, NULL);
    if (FAILED(g_hResult))
    {
        strcpy(g_szErrorMsg, "Couldn't load bitmap file.");
        PostQuitMessage(WM_QUIT);
    }

    g_hResult = g_pDirect3DDevice->GetBackBuffer(0, 0,
        D3DBACKBUFFER_TYPE_MONO, &g_pBackBuffer);
    if (FAILED(g_hResult))
    {
        strcpy(g_szErrorMsg, "Couldn't get back buffer.");
        PostQuitMessage(WM_QUIT);
    }

    return D3D_OK;
}

//////////////////////////////////////////////////////
// CleanUpDirect3D()
//////////////////////////////////////////////////////
void CleanUpDirect3D()
{
    if (g_pBackBuffer)
        g_pBackBuffer->Release();
    if (g_pBitmapSurface)
        g_pBitmapSurface->Release();
    if (g_pDirect3DDevice)
        g_pDirect3DDevice->Release();
    if (g_pDirect3D)
        g_pDirect3D->Release();
}

//////////////////////////////////////////////////////
// Render()
//////////////////////////////////////////////////////
void Render()
{
    PaintBackground();
    g_pDirect3DDevice->Present(NULL, NULL, NULL, NULL);
}

//////////////////////////////////////////////////////
// PaintBackground()
//////////////////////////////////////////////////////
void PaintBackground()
{
    g_pDirect3DDevice->Clear(0, NULL, D3DCLEAR_TARGET, 
        D3DCOLOR_XRGB(0,0,96), 1.0f, 0);
    g_hResult = PlaceRectangle(g_pBackBuffer, g_pBitmapSurface, 0, 2, 1,
        COLUMNSINRECTFILE, DSTCOLUMNCOUNT, RECTSIZE, XOFFSET, YOFFSET);
    g_hResult = PlaceRectangle(g_pBackBuffer, g_pBitmapSurface, 1, 1, 1,
        COLUMNSINRECTFILE, DSTCOLUMNCOUNT, RECTSIZE, XOFFSET, YOFFSET);
    g_hResult = PlaceRectangle(g_pBackBuffer, g_pBitmapSurface, 2, 0, 1,
        COLUMNSINRECTFILE, DSTCOLUMNCOUNT, RECTSIZE, XOFFSET, YOFFSET);
    g_hResult = PlaceRectangle(g_pBackBuffer, g_pBitmapSurface, 3, 2, 0,
        COLUMNSINRECTFILE, DSTCOLUMNCOUNT, RECTSIZE, XOFFSET, YOFFSET);
    g_hResult = PlaceRectangle(g_pBackBuffer, g_pBitmapSurface, 4, 1, 0,
        COLUMNSINRECTFILE, DSTCOLUMNCOUNT, RECTSIZE, XOFFSET, YOFFSET);
    g_hResult = PlaceRectangle(g_pBackBuffer, g_pBitmapSurface, 5, 0, 0,
        COLUMNSINRECTFILE, DSTCOLUMNCOUNT, RECTSIZE, XOFFSET, YOFFSET);
}

//////////////////////////////////////////////////////
// PlaceRectangle()
//////////////////////////////////////////////////////
HRESULT PlaceRectangle(IDirect3DSurface9* pBackBuffer,
    IDirect3DSurface9* pSrcSurface, int iRectNumber,
    int iDstCol, int iDstRow, int iNumRectCols, int iNumDstCols,
    int iRectSize, int iXOffset, int iYOffset)
{
    RECT SrcRect;
    RECT DstRect;
    SrcRect.left = RectNumber2SourceX(iRectNumber,
        iNumRectCols, iRectSize);
    SrcRect.right = SrcRect.left + iRectSize;
    SrcRect.top = RectNumber2SourceY(iRectNumber,
        iNumRectCols, iRectSize);
    SrcRect.bottom = SrcRect.top + iRectSize;
    DstRect.left = Column2X(iDstCol, iRectSize, iNumDstCols) + iXOffset;
    DstRect.top = Row2Y(iDstRow, iRectSize) + iYOffset;
    DstRect.right = DstRect.left + (SrcRect.right - SrcRect.left);
    DstRect.bottom = DstRect.top + (SrcRect.bottom - SrcRect.top);
    
    HRESULT hResult = g_pDirect3DDevice->StretchRect(g_pBitmapSurface,
        &SrcRect, pBackBuffer, &DstRect, D3DTEXF_NONE);
    return hResult;
}

//////////////////////////////////////////////////////
// ColumnRow2RectNum()
//////////////////////////////////////////////////////
int ColumnRow2RectNum(int iCol, int iRow, int iNumCols)
{
    return iRow * iNumCols + iCol;
}

//////////////////////////////////////////////////////
// RectNumber2SourceX()
//////////////////////////////////////////////////////
int RectNumber2SourceX(int iRectNumber, int iNumCols, int iRectSize)
{
    return (iRectNumber - ((int)(iRectNumber / iNumCols))
        * iNumCols) * iRectSize;
}

//////////////////////////////////////////////////////
// RectNumber2SourceY()
//////////////////////////////////////////////////////
int RectNumber2SourceY(int iRectNumber, int iNumCols, int iRectSize)
{
    return ((int)(iRectNumber / iNumCols)) * iRectSize;
}

//////////////////////////////////////////////////////
// Column2X()
//////////////////////////////////////////////////////
int Column2X(int iCol, int iRectSize, int iNumCols)
{
    return (iCol - ((int)((iCol / iNumCols)) *
        iNumCols)) * iRectSize;
}

//////////////////////////////////////////////////////
// Row2Y()
//////////////////////////////////////////////////////
int Row2Y(int iRow, int iRectSize)
{
    return iRow * iRectSize;
}
